#include <gtk/gtk.h>
#include <gdk/gdkdrawable.h>
#include "gdk-pixbuf-rotate.h"
#include <math.h>

#define TEST_FPS 30

GdkPixbuf* test_background = NULL;
GdkPixbuf* test_image = NULL;
GdkPixbuf* test_buffer = NULL;

gboolean test_quit_flag = FALSE;
gdouble mouse_x, mouse_y;

#ifndef M_PI
#define M_PI 3.14159265359
#endif

gboolean test_draw(GtkWidget* area)
{
	GdkRectangle rect;
	static GdkRectangle old_rect;
	static gboolean old_rect_valid = FALSE;
	gint width = gdk_pixbuf_get_width(test_background);
	gint height = gdk_pixbuf_get_height(test_background);

	if (test_quit_flag) return FALSE;

	if (old_rect_valid) {
		gdk_pixbuf_copy_area(test_background, 
			old_rect.x, old_rect.y,
			old_rect.width,
			old_rect.height,
			test_buffer,
			old_rect.x, old_rect.y);
	} else {
		gdk_pixbuf_copy_area(test_background, 0, 0, 
			width, height,
			test_buffer, 0, 0);
	}
	{
		static gdouble angle = 15/180*M_PI;
		gdouble m[4];
		gdouble dx = width/2 - mouse_x;
		gdouble dy = height/2-mouse_y;
		gdouble angle_to_mouse = atan2(dy, dx) - M_PI/2;
		gdouble scale = sqrt(dx*dx + dy*dy)/50 + 0.1;
		gint n_2pi;
		n_2pi = (gint) ((angle_to_mouse - angle + 3*M_PI)/(M_PI*2));
		angle += (n_2pi-1)*2*M_PI;
		angle += (angle_to_mouse-angle)/5;
		gdk_pixbuf_rotate_martix_fill_for_rotation(m, angle);
		gdk_pixbuf_rotate_matrix_mult_number(m, scale);
		gdk_pixbuf_rotate(test_buffer, 200, 200,
			m, test_image,
			gdk_pixbuf_get_width(test_image)/2,
			gdk_pixbuf_get_height(test_image)/2, 
			&rect,
			GDK_INTERP_NEAREST);
	}
	{
		gint x1, x2, y1, y2;
		x1 = MIN(rect.x, old_rect.x);
		x2 = MAX(rect.x+rect.width, old_rect.x+old_rect.width);
		y1 = MIN(rect.y, old_rect.y);
		y2 = MAX(rect.y+rect.height, old_rect.y+old_rect.height);
		gdk_pixbuf_render_to_drawable(test_buffer, 
area->window,
			area->style->fg_gc[GTK_WIDGET_STATE(area)],
			x1, y1, x1, y1, x2-x1, y2-y1,
			GDK_RGB_DITHER_NONE, 0, 0);
	}

	old_rect = rect;
	old_rect_valid = TRUE;

	return TRUE;
}

void test_expose_event(GtkWidget* area, gpointer data)
{
	gdk_pixbuf_render_to_drawable(test_buffer, area->window,
		area->style->fg_gc[GTK_WIDGET_STATE(area)],
		0, 0, 0, 0, -1, -1, GDK_RGB_DITHER_NONE, 0, 0);
}

gint test_quit_event(GtkWidget* widget, gpointer data)
{
	test_quit_flag = TRUE;
	gtk_main_quit();
	return 0;
}

gboolean test_prepare_pixbufs()
{
	GError* error = NULL;
	test_image = gdk_pixbuf_new_from_file("test-img/stock_colorselector_24.png", &error);
	if (error) {
		g_print("cannot load image: %s\n", error->message);
		g_error_free(error);
		return FALSE;
	}
	test_background = gdk_pixbuf_new_from_file("test-img/background.jpg", &error);
	if (error) {
		g_print("cannot load image: %s\n", error->message);
		g_error_free(error);
		return FALSE;
	}
	test_buffer = gdk_pixbuf_new(GDK_COLORSPACE_RGB, FALSE, 8, 400, 400);
	return TRUE;
}

void test_mouse_event(GtkWidget* widget, GdkEventMotion* event)
{
	mouse_x = event->x;
	mouse_y = event->y;
}

void test()
{
	GtkWidget* drawarea;
	GtkWidget* window;

	if (!test_prepare_pixbufs()) return;

	drawarea = gtk_drawing_area_new();
	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);

	g_signal_connect(G_OBJECT(window), "destroy",
		GTK_SIGNAL_FUNC(test_quit_event), NULL);

	g_signal_connect(G_OBJECT(drawarea), "expose_event",
		GTK_SIGNAL_FUNC(test_expose_event), NULL);

	g_signal_connect(G_OBJECT(drawarea), "motion_notify_event",
		GTK_SIGNAL_FUNC(test_mouse_event), NULL);

	gtk_widget_add_events(drawarea, GDK_POINTER_MOTION_MASK);

	gtk_container_add(GTK_CONTAINER(window), drawarea);
	gtk_window_set_default_size(GTK_WINDOW(window), 400, 400);

	mouse_x = 200;
	mouse_y = 200;

	gtk_widget_show_all(window);

	{
		test_draw(drawarea);
		gtk_widget_queue_draw(drawarea);
		g_timeout_add(1000/TEST_FPS, (GSourceFunc) &test_draw, drawarea);
		gtk_main();
	}
}

#ifdef WIN32
#include <windows.h>
int _stdcall
WinMain (struct HINSTANCE__ *hInstance,
	 struct HINSTANCE__ *hPrevInstance,
	 char               *lpszCmdLine,
	 int                 nCmdShow)
{
	gtk_init(&__argc, &__argv);
	test();
	return 0;
}
#else
int main(int argc, char** argv)
{
	gtk_init(&argc, &argv);
	test();
	return 0;
}
#endif

